// CharLib.h - CharLib version 0.1 (28/02/10)
// by Geoffrey White ([my first name]-charlib@randomstuff.org.uk)
//
// DESCRIPTION
// CharLib is a lightweight library for displaying ASCII tiles nicely in a windowed environment, using SDL.
//		* Cross platform C++ code
//		* Full colour ASCII tiles
//		* Scales to fit resizable window
//
// TESTED
//		Demo - Microsoft Windows XP Home Edition, Visual Studio 6.0, SDL 1.2.13
//		Demo - Ubuntu Linux 9.10 (running on VirtualBox), g++ 4.4.1, SDL 1.2.13
//		TODO: more testing, Mac OS X
//
// LICENSE
// I release this library, consisting of the files listed below, as public domain.  There are no restrictions
// on use, copying, redistribution etc, including for profit.  I do not guarantee fitness for purpose.
//		Source\charlib.cpp
//		Source\charlib.h
//		Source\main.cpp				(demo program)
//		Source\resourcescript.rc	(Windows resource script, for embedding the icon into the .exe)
//		Data\icon1.ico				(icon embedded into the .exe)
//		Data\icon32.bmp				(icon loaded by the code)
//		Data\tiles_ascii.bmp		(tile graphics)
//		makefile
// These files were authored by myself, with the exception that data in tiles_ascii.bmp was created with the
// aid of the 'DejaVu Sans Mono' font.  It is my understanding that this kind of content (bitmaps of fonts)
// isn't subject to copyright.
//
// INSTRUCTIONS (Visual Studio)
// To set up a project for the charlib demo:
//		create a Win32 application project / workspace
//		add:
//			charlib.cpp, charlib.h and main.cpp
//			resourcescript.rc
//		the project should also include:
//			SDL-1.2.13\lib\SDL.lib
//			SDL-1.2.13\lib\SDLmain.lib 
//		your include paths should contain:
//			SDL-1.2.13\include
//		finally, change the setting:
//			Project > Settings > All Configurations > C/C++ > category: Code Generation > Use run-time library
//			 > Multithreaded DLL
//		in either the executable directory or windows\system32 should be:
//			sdl.dll
//
// INSTRUCTIONS (g++)
// Go into the project directory and type 'make'
//
// End user builds should be distributed with the SDL runtime package.

#ifndef INCLUDED_CHARLIB
#define INCLUDED_CHARLIB

	// --- compiler settings ---

	#ifdef WIN32
		#pragma warning(disable:4786)
			// sometimes STL related symbols end up rather long and the 'browse information' can't cope.  I don't care.
	#endif

	// --- libs ---

	#include <SDL.h>
	#include <string>

	// --- types ---

	typedef Uint32 u32;
	typedef Uint16 u16;
	typedef Uint8 u8;

	// --- error handling ---

#ifdef _DEBUG
	void char_debug_out(std::string message);
#else
	inline void char_debug_out(std::string message) {}
#endif
	void char_display_error(std::string message); // TODO: make this work better in Linux

	// --- init / update / shutdown ---

	struct char_surface_decl
	{
		std::string file_name;
		int grid_width, grid_height; // number of tiles across and down
	};

	struct char_config
	{
		// app
		std::string window_title;
		std::string icon_file_name;

		// source tiles
		char_surface_decl ascii_decl;		// containing ascii characters 32 .. 127

		// grid display
		int grid_width, grid_height;		// width and height of the display grid in tiles
		int aspect_width, aspect_height;	// (1, 1 if you want square tiles)
	};

	void char_init(const char_config &config);
	bool char_update(); // process stuff, returns false if the user has chosen to quit
	void char_shutdown();

	// --- drawing ---
	//
	// (actually carrying out these instructions is deferred until 'char_update')

	struct char_style
	{
		int fore_r, fore_g, fore_b; // foreground colour (white on the tile becomes this)
		int back_r, back_b, back_g; // background colour (black on the tile becomes this)
	};
	extern char_style char_default_style;

	void char_clear();
	void char_draw(int x, int y, char ch, const char_style &style);
	void char_draw(int x, int y, const std::string &str, const char_style &style);

	// --- input ---

	struct char_key
	{
		char ascii;
		SDLKey sdl;
	};

	bool char_get_key(char_key &key);
		// returns true if a key was pressed since the last query, and puts the details
		// into the parameter (non-ascii keys will return ascii code 0)

#endif